%% DEMO 1
% This script performs the synthetic experiment with low noise 
%
% What needs to be set:
% name_of_signal - name of signal to be loaded
% loaded signal = signal_characteristic - a structrure which includes
%   - N - length of signal
%   - degree - polynomial degree
%   - y - noisy signal
%   - signal_clean - signal without noise
%   - sigma - noise standard deviation
%   - coeff - parameterization coefficients of signal without noise
% parameters of solver 
%   - MAX_ITER - maximum number of iterations
% parameters for methods (param_method)
%   - A - representation matrix
%   - weights of differences - Default: vector of ones
%   - tau_x0, tau_x1 and tau_x2
% parameters for breakpoint detection (param_method)
%   - threshold - for breakpoint detection
%   - window_length - length of window in median filter
%   - p - type of norm. Default p=2 -> Euclidean norm.
%   - proximity - smallest proximity of two breakpoints. Default
%   proximity=2
%--------------------------------------------------------------------------
 
clear variables
close all

%% Load signal 
name_of_signal='DEMO_1_data.mat';
load(name_of_signal)

% properties of the loaded signal
degree=signal_characteristics.degree;
N=signal_characteristics.N; %length of the signal
y=signal_characteristics.y;
signal_clean=signal_characteristics.signal_clean;
sigma=signal_characteristics.sigma;
parameterization_coeff=signal_characteristics.coeff;

% Compute SNR of observed signal y
y_error=y-signal_clean;
SNR_dB_orig = 20*log10(norm(signal_clean)/norm(y_error));

%% Plot signal and its parameterization
handle_signal = figure('Name','Signal','NumberTitle','off');
plot(1:N,signal_clean);
hold on;
plot(1:N,y,'r');
title(['SNR = ', num2str(SNR_dB_orig), ' dB'])
legend('clean', 'observed');

handle_params = figure('Name','Parameterization coefficients','NumberTitle','off');
plot(1:N,parameterization_coeff(:,1),'bx')
hold on
plot(1:N,parameterization_coeff(:,2),'bo')
hold on
plot(1:N,parameterization_coeff(:,3),'b*')

legend('x0', 'x1', 'x2');

%% Parameters setting
% parameters of solver
MAX_ITER=2000;
param_solver.verbose = 0; % display parameter
param_solver.maxit = MAX_ITER;       % maximum number of iterations
param_solver.tol = 1e-6;        % tolerance to stop iterating
param_solver.debug_mode = 1;

% tau - there is that many taus to be set as is the degree of polynomial signal
% sigma is loaded
tau_x0 = sigma*5; 
tau_x1 = sigma*3.5;
tau_x2 = sigma*2.5; 
% vector of used taus
vector_tau=[tau_x0, tau_x1, tau_x2];

% Representation matrix
A=[];
for i=1:degree
    d=(1:N)/N;
    d=d.^(i-1);
    D=diag(d);
    A=[A,D];
    
end

A = sparse(A);  %making it a bit faster with no effort

% save in param_method
param_method.A=A;
param_method.tau=vector_tau;

% structure for saving results of used methods
batch = cell(0,0);
batch_cnt = 0;

%% Solve using CP algorithm
[batch,batch_cnt] = lib_CP(param_method, param_solver, y , N, degree, batch, batch_cnt); 
% recomputed SNR of achieved signal
y_error = batch{batch_cnt}.y_recon-signal_clean;
SNR_dB_CP = 20*log10(norm(signal_clean)/norm(y_error));

%% Plot results
colors = 'gcmyr'; %colors for succesive plots
letters = 'xo*.';

% Display signal
figure(handle_signal);
for cnt = 1:batch_cnt
    plot(batch{cnt}.y_recon, colors(cnt));
end
legend('clean','observed', 'CP');

% Display parameters
figure(handle_params); 
for cnt = 1:batch_cnt
    for i=1:degree
        plot(batch{cnt}.sol(1+N*(i-1):N*i), [colors(cnt) letters(i)]);
    end
end
axis tight
legend('x0', 'x1', 'x2', 'x0 CP', 'x1 CP','x2 CP');
xlabel('Samples')
ylabel('Values of parametrization coefficients')
%% Breakpoint detection and signal denoising
% set parameters
param_method.threshold=0.1;
param_method.window_length=5;  %window length for the median filter
% breakpoints detection and signal denoising using least squares
[batch, batch_cnt]=lib_signal_denoising(y, batch, batch_cnt,  N, degree, param_method);
disp(['Breakpoints in the model: ', num2str(find(diff(parameterization_coeff(:,1))~=0)')])

% recomputing SNR of denoised signal
y_error=batch{cnt}.signal_recon-signal_clean;
SNR_dB_recon = 20*log10(norm(signal_clean)/norm(y_error));
batch{cnt}.SNR_dB_recon = SNR_dB_recon;

% display denoised signal and its parameters
% display signal
figure('Name','Reconstructed signal after LS fit','NumberTitle','off');
pom_max=max(y);
pom_min=min(y);

for int=1:2:length(batch{cnt}.breakpoints)-1
    hight=pom_max-pom_min;
    y_min=pom_min-0.05*hight;
    y_max=hight+0.1*hight;
    h=rectangle('Position',[batch{cnt}.breakpoints(int)+0.5,y_min,batch{cnt}.breakpoints(int+1)-batch{cnt}.breakpoints(int),y_max]);
    set(h,'FaceColor',[.95 .95 .95])
    set(h,'EdgeColor','none')
    hold on
end
xlabel('Samples')
ylabel('Signals values')

plot(y,'r')
hold on
plot(signal_clean)
hold on
for cnt = 1:batch_cnt
    plot(batch{cnt}.signal_recon, colors(cnt));
    hold on
end

set (gca, 'YLim', [y_min,y_max+y_min])
legend('signal noisy', 'signal clean', 'signal recon CP + LS');

% display parameters
figure('Name','Parameterization coefficients after LS fit','NumberTitle','off'); 

for cnt = 1:batch_cnt
    pom_max=max(max(batch{cnt}.recon));
    pom_min=min(min(batch{cnt}.recon));
end    

for int=1:2:length(batch{cnt}.breakpoints)-1
    hight=pom_max-pom_min;
    y_min=pom_min-0.05*hight;
    y_max=hight+0.1*hight;
    h=rectangle('Position',[batch{cnt}.breakpoints(int)+0.5,y_min,batch{cnt}.breakpoints(int+1)-batch{cnt}.breakpoints(int),y_max]);
    set(h,'FaceColor',[.95 .95 .95])
    set(h,'EdgeColor','none')
    hold on
end
for cnt = 1:batch_cnt
    for i=1:degree
        plot(batch{cnt}.recon(:,i), [colors(cnt), letters(i)]);
        hold on
    end
end

for i=1:degree
    plot (parameterization_coeff(:,i),['b',letters(i)])
    hold on
end

set (gca, 'YLim', [y_min,y_max+y_min])
legend('x0', 'x1', 'x2','x0 recon CP', 'x1 recon CP', 'x2 recon CP');
xlabel('Samples')
ylabel('Values of parametrization coefficients')
